@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tk.h,v 1.24.2.1 1999/05/19 23:24:26 redman Exp $
+ * RCS: @(#) $Id: tk.h,v 1.30 1999/08/10 23:16:42 redman Exp $
  */
 
 #ifndef _TK
@@ -32,6 +32,7 @@ extern "C" {
  *
  * unix/configure.in
  * README
+ * win/configure.in
  * win/makefile.vc	(only if major.minor changes, not patchlevel)
  * library/tk.tcl	(only if major.minor changes, not patchlevel)
  * mac/README		(only if major.minor changes, not patchlevel)
@@ -43,12 +44,12 @@ extern "C" {
  */
 
 #define TK_MAJOR_VERSION   8
-#define TK_MINOR_VERSION   1
+#define TK_MINOR_VERSION   2
 #define TK_RELEASE_LEVEL   TCL_FINAL_RELEASE
-#define TK_RELEASE_SERIAL  1
+#define TK_RELEASE_SERIAL  0
 
-#define TK_VERSION "8.1"
-#define TK_PATCH_LEVEL "8.1.1+"
+#define TK_VERSION "8.2"
+#define TK_PATCH_LEVEL "8.2.0+"
 
 /*
  * The following definitions set up the proper options for Macintosh
@@ -1171,9 +1172,15 @@ EXTERN void	Tk_SetTileCanvasItem _ANSI_ARGS_((Tk_Tile tile,
  */
 
 typedef struct Tk_ImageType Tk_ImageType;
+#ifdef USE_OLD_IMAGE
+typedef int (Tk_ImageCreateProc) _ANSI_ARGS_((Tcl_Interp *interp,
+	char *name, int argc, char **argv, Tk_ImageType *typePtr,
+	Tk_ImageMaster master, ClientData *masterDataPtr));
+#else
 typedef int (Tk_ImageCreateProc) _ANSI_ARGS_((Tcl_Interp *interp,
 	char *name, int objc, Tcl_Obj *CONST objv[], Tk_ImageType *typePtr,
 	Tk_ImageMaster master, ClientData *masterDataPtr));
+#endif
 typedef ClientData (Tk_ImageGetProc) _ANSI_ARGS_((Tk_Window tkwin,
 	ClientData masterData));
 typedef void (Tk_ImageDisplayProc) _ANSI_ARGS_((ClientData instanceData,
@@ -1265,6 +1272,24 @@ typedef struct Tk_PhotoImageBlock {
  */
 
 typedef struct Tk_PhotoImageFormat Tk_PhotoImageFormat;
+#ifdef USE_OLD_IMAGE
+typedef int (Tk_ImageFileMatchProc) _ANSI_ARGS_((Tcl_Channel chan,
+	char *fileName, char *formatString, int *widthPtr, int *heightPtr));
+typedef int (Tk_ImageStringMatchProc) _ANSI_ARGS_((char *string,
+	char *formatString, int *widthPtr, int *heightPtr));
+typedef int (Tk_ImageFileReadProc) _ANSI_ARGS_((Tcl_Interp *interp,
+	Tcl_Channel chan, char *fileName, char *formatString,
+	Tk_PhotoHandle imageHandle, int destX, int destY,
+	int width, int height, int srcX, int srcY));
+typedef int (Tk_ImageStringReadProc) _ANSI_ARGS_((Tcl_Interp *interp,
+	char *string, char *formatString, Tk_PhotoHandle imageHandle,
+	int destX, int destY, int width, int height, int srcX, int srcY));
+typedef int (Tk_ImageFileWriteProc) _ANSI_ARGS_((Tcl_Interp *interp,
+	char *fileName, char *formatString, Tk_PhotoImageBlock *blockPtr));
+typedef int (Tk_ImageStringWriteProc) _ANSI_ARGS_((Tcl_Interp *interp,
+	Tcl_DString *dataPtr, char *formatString,
+	Tk_PhotoImageBlock *blockPtr));
+#else
 typedef int (Tk_ImageFileMatchProc) _ANSI_ARGS_((Tcl_Channel chan,
 	char *fileName, Tcl_Obj *format, int *widthPtr,
 	int *heightPtr, Tcl_Interp *interp));
@@ -1283,6 +1308,7 @@ typedef int (Tk_ImageFileWriteProc) _ANSI_ARGS_((Tcl_Interp *interp,
 typedef int (Tk_ImageStringWriteProc) _ANSI_ARGS_((Tcl_Interp *interp,
 	Tcl_DString *dataPtr, Tcl_Obj *format,
 	Tk_PhotoImageBlock *blockPtr));
+#endif
 
 /*
  * The following structure represents a particular file format for
@@ -1317,6 +1343,17 @@ struct Tk_PhotoImageFormat {
 				 * currently known.  Filled in by Tk, not
 				 * by image format handler. */
 };
+
+EXTERN void		Tk_CreateImageType _ANSI_ARGS_((
+				Tk_ImageType * typePtr));
+EXTERN void		Tk_CreatePhotoImageFormat _ANSI_ARGS_((
+				Tk_PhotoImageFormat * formatPtr));
+
+#ifndef USE_OLD_IMAGE
+#define Tk_CreateImageType Tk_CreateNewImageType
+#define Tk_CreatePhotoImageFormat Tk_CreateNewPhotoImageFormat
+#endif
+
 
 /*
  *--------------------------------------------------------------
