@@ -6,12 +6,12 @@
  *	procedures used for generating Postscript.
  *
  * Copyright (c) 1991-1994 The Regents of the University of California.
- * Copyright (c) 1994-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkCanvPs.c,v 1.4 1998/09/22 18:57:16 stanton Exp $
+ * RCS: @(#) $Id: tkCanvPs.c,v 1.5 1999/04/16 01:51:11 stanton Exp $
  */
 
 #include "tkInt.h"
@@ -119,6 +119,7 @@ static Tk_ConfigSpec configSpecs[] = {
  * The prolog data. Generated by str2c from prolog.ps
  * This was split in small chunks by str2c because
  * some C compiler have limitations on the size of static strings.
+ * (str2c is a small tcl script in tcl's tool directory (source release))
  */
 static CONST char * CONST  prolog[]= {
 	/* Start of part 1 (2000 characters) */
@@ -127,7 +128,7 @@ static CONST char * CONST  prolog[]= {
 \n\
 % This is a standard prolog for Postscript generated by Tk's canvas\n\
 % widget.\n\
-% RCS: @(#) $Id: tkCanvPs.c,v 1.4 1998/09/22 18:57:16 stanton Exp $\n\
+% RCS: @(#) $Id: tkCanvPs.c,v 1.5 1999/04/16 01:51:11 stanton Exp $\n\
 \n\
 % The definitions below just define all of the variables used in\n\
 % any of the procedures here.  This is needed for obscure reasons\n\
@@ -718,20 +719,20 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
     if (psInfo.prolog) {
     Tcl_AppendResult(interp, "%!PS-Adobe-3.0 EPSF-3.0\n",
 	    "%%Creator: Tk Canvas Widget\n", (char *) NULL);
-#if !(defined(__WIN32__) || defined(MAC_TCL))
+#ifdef HAVE_PW_GECOS
     if (!Tcl_IsSafe(interp)) {
-	struct passwd *pwPtr = getpwuid(getuid());
+	struct passwd *pwPtr = getpwuid(getuid());	/* INTL: Native. */
 	Tcl_AppendResult(interp, "%%For: ",
 		(pwPtr != NULL) ? pwPtr->pw_gecos : "Unknown", "\n",
 		(char *) NULL);
 	endpwent();
     }
-#endif /* __WIN32__ || MAC_TCL */
+#endif /* HAVE_PW_GECOS */
     Tcl_AppendResult(interp, "%%Title: Window ",
 	    Tk_PathName(tkwin), "\n", (char *) NULL);
     time(&now);
     Tcl_AppendResult(interp, "%%CreationDate: ",
-	    ctime(&now), (char *) NULL);
+	    ctime(&now), (char *) NULL);		/* INTL: Native. */
     if (!psInfo.rotate) {
 	sprintf(string, "%d %d %d %d",
 		(int) (psInfo.pageX + psInfo.scale*deltaX),
@@ -772,8 +773,8 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
     }
 
     if (psInfo.chan != NULL) {
-	Tcl_Write(psInfo.chan, interp->result, -1);
-	Tcl_ResetResult(interp);
+	Tcl_Write(psInfo.chan, Tcl_GetStringResult(interp), -1);
+	Tcl_ResetResult(canvasPtr->interp);
     }
 
     /*
@@ -824,8 +825,8 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 	" lineto closepath clip newpath\n", (char *) NULL);
     }
     if (psInfo.chan != NULL) {
-	Tcl_Write(psInfo.chan, interp->result, -1);
-	Tcl_ResetResult(interp);
+	Tcl_Write(psInfo.chan, Tcl_GetStringResult(interp), -1);
+	Tcl_ResetResult(canvasPtr->interp);
     }
 
     /*
@@ -852,7 +853,7 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 	result = (*itemPtr->typePtr->postscriptProc)(interp,
 		(Tk_Canvas) canvasPtr, itemPtr, 0);
 	if (result != TCL_OK) {
-	    char msg[100];
+	    char msg[64 + TCL_INTEGER_SPACE];
 
 	    sprintf(msg, "\n    (generating Postscript for item %d)",
 		    itemPtr->id);
@@ -861,7 +862,7 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 	}
 	Tcl_AppendResult(interp, "grestore\n", (char *) NULL);
 	if (psInfo.chan != NULL) {
-	    Tcl_Write(psInfo.chan, interp->result, -1);
+	    Tcl_Write(psInfo.chan, Tcl_GetStringResult(interp), -1);
 	    Tcl_ResetResult(interp);
 	}
     }
@@ -874,12 +875,12 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
      */
 
     if (psInfo.prolog) {
-	Tcl_AppendResult(interp, "restore showpage\n\n",
-		"%%Trailer\nend\n%%EOF\n", (char *) NULL);
+      Tcl_AppendResult(interp, "restore showpage\n\n",
+	    "%%Trailer\nend\n%%EOF\n", (char *) NULL);
     }
     if (psInfo.chan != NULL) {
-	Tcl_Write(psInfo.chan, interp->result, -1);
-	Tcl_ResetResult(interp);
+	Tcl_Write(psInfo.chan, Tcl_GetStringResult(interp), -1);
+	Tcl_ResetResult(canvasPtr->interp);
     }
 
     /*
@@ -934,9 +935,9 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
  *
  * Results:
  *	Returns a standard Tcl return value.  If an error occurs
- *	then an error message will be left in interp->result.
+ *	then an error message will be left in the interp's result.
  *	If no error occurs, then additional Postscript will be
- *	appended to interp->result.
+ *	appended to the interp's result.
  *
  * Side effects:
  *	None.
@@ -1013,9 +1014,9 @@ Tk_PostscriptColor(interp, psInfo, colorPtr)
  *
  * Results:
  *	Returns a standard Tcl return value.  If an error occurs
- *	then an error message will be left in interp->result.
+ *	then an error message will be left in the interp's result.
  *	If no error occurs, then additional Postscript will be
- *	appended to the interp->result.
+ *	appended to the interp's result.
  *
  * Side effects:
  *	The Postscript font name is entered into psInfoPtr->fontTable
@@ -1033,7 +1034,7 @@ Tk_PostscriptFont(interp, psInfo, tkfont)
 {
     TkPostscriptInfo *psInfoPtr = (TkPostscriptInfo *) psInfo;
     char *end;
-    char pointString[20];
+    char pointString[TCL_INTEGER_SPACE];
     Tcl_DString ds;
     int i, points;
 
@@ -1105,9 +1106,9 @@ Tk_PostscriptFont(interp, psInfo, tkfont)
  *
  * Results:
  *	Returns a standard Tcl return value.  If an error occurs
- *	then an error message will be left in interp->result.
+ *	then an error message will be left in the interp's result.
  *	If no error occurs, then additional Postscript will be
- *	appended to interp->result.
+ *	appended to the interp's result.
  *
  * Side effects:
  *	None.
@@ -1204,9 +1205,9 @@ Tk_PostscriptBitmap(interp, tkwin, psInfo, bitmap, startX, startY, width,
  *
  * Results:
  *	Returns a standard Tcl return value.  If an error occurs
- *	then an error message will be left in interp->result.
+ *	then an error message will be left in the interp's result.
  *	If no error occurs, then additional Postscript will be
- *	appended to interp->result.
+ *	appended to the interp's result.
  *
  * Side effects:
  *	None.
@@ -1224,7 +1225,7 @@ Tk_PostscriptStipple(interp, tkwin, psInfo, bitmap)
 {
     TkPostscriptInfo *psInfoPtr = (TkPostscriptInfo *) psInfo;
     int width, height;
-    char string[100];
+    char string[TCL_INTEGER_SPACE * 2];
     Window dummyRoot;
     int dummyX, dummyY;
     unsigned dummyBorderwidth, dummyDepth;
@@ -1291,7 +1292,7 @@ Tk_PostscriptY(y, psInfo)
  *	commands to create the path.
  *
  * Results:
- *	Postscript commands get appended to what's in interp->result.
+ *	Postscript commands get appended to what's in the interp's result.
  *
  * Side effects:
  *	None.
@@ -1339,7 +1340,7 @@ Tk_PostscriptPath(interp, psInfo, coordPtr, numPoints)
  *	TCL_OK is returned, then everything went well and the
  *	screen distance is stored at *doublePtr;  otherwise
  *	TCL_ERROR is returned and an error message is left in
- *	interp->result.
+ *	the interp's result.
  *
  * Side effects:
  *	None.
@@ -1396,7 +1397,7 @@ GetPostscriptPoints(interp, string, doublePtr)
     *doublePtr = d;
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1425,19 +1426,19 @@ TkImageGetColor(cdata, pixel, red, green, blue)
     double *red, *green, *blue;         /* Color data to return */
 {
     if (cdata->separated) {
-        int r = (pixel & cdata->red_mask) >> cdata->red_shift;
-        int g = (pixel & cdata->green_mask) >> cdata->green_shift;
-        int b = (pixel & cdata->blue_mask) >> cdata->blue_shift;
-        *red = cdata->colors[r].red / 65535.0;
-        *green = cdata->colors[g].green / 65535.0;
-        *blue = cdata->colors[b].blue / 65535.0;
+	int r = (pixel & cdata->red_mask) >> cdata->red_shift;
+	int g = (pixel & cdata->green_mask) >> cdata->green_shift;
+	int b = (pixel & cdata->blue_mask) >> cdata->blue_shift;
+	*red = cdata->colors[r].red / 65535.0;
+	*green = cdata->colors[g].green / 65535.0;
+	*blue = cdata->colors[b].blue / 65535.0;
     } else {
-        *red = cdata->colors[pixel].red / 65535.0;
-        *green = cdata->colors[pixel].green / 65535.0;
-        *blue = cdata->colors[pixel].blue / 65535.0;
+	*red = cdata->colors[pixel].red / 65535.0;
+	*green = cdata->colors[pixel].green / 65535.0;
+	*blue = cdata->colors[pixel].blue / 65535.0;
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1528,6 +1529,7 @@ TkPostscriptImage(interp, tkwin, psInfo, ximage, x, y, width, height)
     else
 	cdata.color = 1;
 
+
     XQueryColors(Tk_Display(tkwin), cmap, cdata.colors, ncolors);
 
     /*
@@ -1537,11 +1539,11 @@ TkPostscriptImage(interp, tkwin, psInfo, ximage, x, y, width, height)
      */
 
     if (!cdata.color && level == 2) {
-        level = 1;
+	level = 1;
     }
 
     if (!cdata.color && cdata.ncolors == 2) {
-        level = 0;
+	level = 0;
     }
 
     /*
@@ -1549,132 +1551,132 @@ TkPostscriptImage(interp, tkwin, psInfo, ximage, x, y, width, height)
      * with a string less than 64 KB long (this is a limit in the 
      * Postscript interpreter).
      */
-    
+
     switch (level)
     {
-        case 0: bytesPerLine = (width + 7) / 8;  maxWidth = 240000;  break;
-        case 1: bytesPerLine = width;  maxWidth = 60000;  break;
-        case 2: bytesPerLine = 3 * width;  maxWidth = 20000;  break;
+	case 0: bytesPerLine = (width + 7) / 8;  maxWidth = 240000;  break;
+	case 1: bytesPerLine = width;  maxWidth = 60000;  break;
+	case 2: bytesPerLine = 3 * width;  maxWidth = 20000;  break;
     }
 
     if (bytesPerLine > 60000) {
-        Tcl_ResetResult(interp);
-        sprintf(buffer,
-                "Can't generate Postscript for images more than %d pixels wide",
-                maxWidth);
-        Tcl_AppendResult(interp, buffer, (char *) NULL);
+	Tcl_ResetResult(interp);
+	sprintf(buffer,
+		"Can't generate Postscript for images more than %d pixels wide",
+		maxWidth);
+	Tcl_AppendResult(interp, buffer, (char *) NULL);
 	ckfree((char *) cdata.colors);
-        return TCL_ERROR;
+	return TCL_ERROR;
     }
 
     maxRows = 60000 / bytesPerLine;
 
     for (band = height-1; band >= 0; band -= maxRows) {
-        int rows = (band >= maxRows) ? maxRows : band + 1;
-        int lineLen = 0;
-        switch (level) {
-            case 0:
-                sprintf(buffer, "%d %d 1 matrix {\n<", width, rows);
-                Tcl_AppendResult(interp, buffer, (char *) NULL);
-                break;
-            case 1:
-                sprintf(buffer, "%d %d 8 matrix {\n<", width, rows);
-                Tcl_AppendResult(interp, buffer, (char *) NULL);
-                break;
-            case 2:
-                sprintf(buffer, "%d %d 8 matrix {\n<",
-                        width, rows);
-                Tcl_AppendResult(interp, buffer, (char *) NULL);
-                break;
-        }
-        for (yy = band; yy > band - rows; yy--) {
-            switch (level) {
-                case 0: {
-                    /*
-                     * Generate data for image in monochrome mode.
-                     * No attempt at dithering is made--instead, just
-                     * set a threshold.
-                     */
-                    unsigned char mask=0x80;
-                    unsigned char data=0x00;
-                    for (xx = x; xx< x+width; xx++) {
-                        TkImageGetColor(&cdata, XGetPixel(ximage, xx, yy),
-                                        &red, &green, &blue);
-                        if (0.30 * red + 0.59 * green + 0.11 * blue > 0.5)
-                            data |= mask;
-                        mask >>= 1;
-                        if (mask == 0) {
-                            sprintf(buffer, "%02X", data);
-                            Tcl_AppendResult(interp, buffer, (char *) NULL);
-                            lineLen += 2;
-                            if (lineLen > 60) {
-                                lineLen = 0;
-                                Tcl_AppendResult(interp, "\n", (char *) NULL);
-                            }
-                            mask=0x80;
-                            data=0x00;
-                        }
-                    }
-                    if ((width % 8) != 0) {
-                        sprintf(buffer, "%02X", data);
-                        Tcl_AppendResult(interp, buffer, (char *) NULL);
-                        mask=0x80;
-                        data=0x00;
-                    }
-                    break;
-                }
-                case 1: {
-                    /*
-                     * Generate data in gray mode--in this case, take a 
-                     * weighted sum of the red, green, and blue values.
-                     */
-                    for (xx = x; xx < x+width; xx ++) {
-                        TkImageGetColor(&cdata, XGetPixel(ximage, xx, yy),
-                                        &red, &green, &blue);
-                        sprintf(buffer, "%02X", (int) floor(0.5 + 255.0 *
-                                                            (0.30 * red +
-                                                             0.59 * green +
-                                                             0.11 * blue)));
-                        Tcl_AppendResult(interp, buffer, (char *) NULL);
-                        lineLen += 2;
-                        if (lineLen > 60) {
-                            lineLen = 0;
-                            Tcl_AppendResult(interp, "\n", (char *) NULL);
-                        }
-                    }
-                    break;
-                }
-                case 2: {
-                    /*
-                     * Finally, color mode.  Here, just output the red, green,
-                     * and blue values directly.
-                     */
-                    for (xx = x; xx < x+width; xx++) {
-                        TkImageGetColor(&cdata, XGetPixel(ximage, xx, yy),
-                                        &red, &green, &blue);
-                        sprintf(buffer, "%02X%02X%02X",
-                                (int) floor(0.5 + 255.0 * red),
-                                (int) floor(0.5 + 255.0 * green),
-                                (int) floor(0.5 + 255.0 * blue));
-                        Tcl_AppendResult(interp, buffer, (char *) NULL);
-                        lineLen += 6;
-                        if (lineLen > 60) {
-                            lineLen = 0;
-                            Tcl_AppendResult(interp, "\n", (char *) NULL);
-                        }
-                    }
-                    break;
-                }
-            }
-        }
-        switch (level) {
-            case 0: sprintf(buffer, ">\n} image\n"); break;
-            case 1: sprintf(buffer, ">\n} image\n"); break;
-            case 2: sprintf(buffer, ">\n} false 3 colorimage\n"); break;
-        }
-        Tcl_AppendResult(interp, buffer, (char *) NULL);
-        sprintf(buffer, "0 %d translate\n", rows);
-        Tcl_AppendResult(interp, buffer, (char *) NULL);
+	int rows = (band >= maxRows) ? maxRows : band + 1;
+	int lineLen = 0;
+	switch (level) {
+	    case 0:
+		sprintf(buffer, "%d %d 1 matrix {\n<", width, rows);
+		Tcl_AppendResult(interp, buffer, (char *) NULL);
+		break;
+	    case 1:
+		sprintf(buffer, "%d %d 8 matrix {\n<", width, rows);
+		Tcl_AppendResult(interp, buffer, (char *) NULL);
+		break;
+	    case 2:
+		sprintf(buffer, "%d %d 8 matrix {\n<",
+			width, rows);
+		Tcl_AppendResult(interp, buffer, (char *) NULL);
+		break;
+	}
+	for (yy = band; yy > band - rows; yy--) {
+	    switch (level) {
+		case 0: {
+		    /*
+		     * Generate data for image in monochrome mode.
+		     * No attempt at dithering is made--instead, just
+		     * set a threshold.
+		     */
+		    unsigned char mask=0x80;
+		    unsigned char data=0x00;
+		    for (xx = x; xx< x+width; xx++) {
+			TkImageGetColor(&cdata, XGetPixel(ximage, xx, yy),
+					&red, &green, &blue);
+			if (0.30 * red + 0.59 * green + 0.11 * blue > 0.5)
+			    data |= mask;
+			mask >>= 1;
+			if (mask == 0) {
+			    sprintf(buffer, "%02X", data);
+			    Tcl_AppendResult(interp, buffer, (char *) NULL);
+			    lineLen += 2;
+			    if (lineLen > 60) {
+			        lineLen = 0;
+			        Tcl_AppendResult(interp, "\n", (char *) NULL);
+			    }
+			    mask=0x80;
+			    data=0x00;
+			}
+		    }
+		    if ((width % 8) != 0) {
+		        sprintf(buffer, "%02X", data);
+		        Tcl_AppendResult(interp, buffer, (char *) NULL);
+		        mask=0x80;
+		        data=0x00;
+		    }
+		    break;
+		}
+		case 1: {
+		    /*
+		     * Generate data in gray mode--in this case, take a 
+		     * weighted sum of the red, green, and blue values.
+		     */
+		    for (xx = x; xx < x+width; xx ++) {
+			TkImageGetColor(&cdata, XGetPixel(ximage, xx, yy),
+					&red, &green, &blue);
+			sprintf(buffer, "%02X", (int) floor(0.5 + 255.0 *
+							    (0.30 * red +
+							     0.59 * green +
+							     0.11 * blue)));
+			Tcl_AppendResult(interp, buffer, (char *) NULL);
+			lineLen += 2;
+			if (lineLen > 60) {
+			    lineLen = 0;
+			    Tcl_AppendResult(interp, "\n", (char *) NULL);
+			}
+		    }
+		    break;
+		}
+		case 2: {
+		    /*
+		     * Finally, color mode.  Here, just output the red, green,
+		     * and blue values directly.
+		     */
+			for (xx = x; xx < x+width; xx++) {
+			TkImageGetColor(&cdata, XGetPixel(ximage, xx, yy),
+				&red, &green, &blue);
+			sprintf(buffer, "%02X%02X%02X",
+				(int) floor(0.5 + 255.0 * red),
+				(int) floor(0.5 + 255.0 * green),
+				(int) floor(0.5 + 255.0 * blue));
+			Tcl_AppendResult(interp, buffer, (char *) NULL);
+			lineLen += 6;
+			if (lineLen > 60) {
+			    lineLen = 0;
+			    Tcl_AppendResult(interp, "\n", (char *) NULL);
+			}
+		    }
+		    break;
+		}
+	    }
+	}
+	switch (level) {
+	    case 0: sprintf(buffer, ">\n} image\n"); break;
+	    case 1: sprintf(buffer, ">\n} image\n"); break;
+	    case 2: sprintf(buffer, ">\n} false 3 colorimage\n"); break;
+	}
+	Tcl_AppendResult(interp, buffer, (char *) NULL);
+	sprintf(buffer, "0 %d translate\n", rows);
+	Tcl_AppendResult(interp, buffer, (char *) NULL);
     }
     ckfree((char *) cdata.colors);
     return TCL_OK;
