@@ -10,12 +10,14 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkConsole.c 1.54 97/10/17 10:46:08
+ * RCS: @(#) $Id: tkConsole.c,v 1.10 1999/07/30 02:09:25 redman Exp $
  */
 
 #include "tk.h"
 #include <string.h>
 
+#include "tkInt.h"
+
 /*
  * A data structure of the following type holds information for each console
  * which a handler (i.e. a Tcl command) has been defined for a particular
@@ -27,7 +29,17 @@ typedef struct ConsoleInfo {
     Tcl_Interp *interp;		/* Interpreter to send console commands. */
 } ConsoleInfo;
 
-static Tcl_Interp *gStdoutInterp = NULL;
+typedef struct ThreadSpecificData {
+    Tcl_Interp *gStdoutInterp;
+} ThreadSpecificData;
+static Tcl_ThreadDataKey dataKey;
+static int consoleInitialized = 0;
+
+/* 
+ * The Mutex below is used to lock access to the consoleIntialized flag
+ */
+
+TCL_DECLARE_MUTEX(consoleMutex)
 
 /*
  * Forward declarations for procedures defined later in this file:
@@ -35,8 +47,6 @@ static Tcl_Interp *gStdoutInterp = NULL;
  * The first three will be used in the tk app shells...
  */
  
-void	TkConsoleCreate _ANSI_ARGS_((void));
-int	TkConsoleInit _ANSI_ARGS_((Tcl_Interp *interp));
 void	TkConsolePrint _ANSI_ARGS_((Tcl_Interp *interp,
 			    int devId, char *buffer, long size));
 
@@ -75,11 +85,111 @@ static Tcl_ChannelType consoleChannelType = {
     ConsoleWatch,		/* Watch for events on console. */
     ConsoleHandle,		/* Get a handle from the device. */
 };
+
+
+#ifdef __WIN32__
+
+#include <windows.h>
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * ShouldUseConsoleChannel
+ *
+ * 	Check to see if console window should be used for a given
+ *      standard channel
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Creates the console channel and installs it as the standard
+ *	channels.
+ *
+ *----------------------------------------------------------------------
+ */
+static int ShouldUseConsoleChannel(type)
+    int type;
+{
+    DWORD handleId;		/* Standard handle to retrieve. */
+    DCB dcb;
+    DWORD consoleParams;
+    DWORD fileType;
+    int mode;
+    char *bufMode;
+    HANDLE handle;
+
+    switch (type) {
+	case TCL_STDIN:
+	    handleId = STD_INPUT_HANDLE;
+	    mode = TCL_READABLE;
+	    bufMode = "line";
+	    break;
+	case TCL_STDOUT:
+	    handleId = STD_OUTPUT_HANDLE;
+	    mode = TCL_WRITABLE;
+	    bufMode = "line";
+	    break;
+	case TCL_STDERR:
+	    handleId = STD_ERROR_HANDLE;
+	    mode = TCL_WRITABLE;
+	    bufMode = "none";
+	    break;
+	default:
+	    return 0;
+	    break;
+    }
+
+    handle = GetStdHandle(handleId);
+
+    /*
+     * Note that we need to check for 0 because Windows will return 0 if this
+     * is not a console mode application, even though this is not a valid
+     * handle. 
+     */
+
+    if ((handle == INVALID_HANDLE_VALUE) || (handle == 0)) {
+	return 1;
+    }
+    fileType = GetFileType(handle);
+
+    /*
+     * If the file is a character device, we need to try to figure out
+     * whether it is a serial port, a console, or something else.  We
+     * test for the console case first because this is more common.
+     */
+
+    if (fileType == FILE_TYPE_CHAR) {
+	dcb.DCBlength = sizeof( DCB ) ;
+	if (!GetConsoleMode(handle, &consoleParams) &&
+		!GetCommState(handle, &dcb)) {
+	    /*
+	     * Don't use a CHAR type channel for stdio, otherwise Tk
+	     * runs into trouble with the MS DevStudio debugger.
+	     */
+	    
+	    return 1;
+	}
+    } else if (fileType == FILE_TYPE_UNKNOWN) {
+	return 1;
+    } else if (Tcl_GetStdChannel(type) == NULL) {
+	return 1;
+    }
+
+    return 0;
+}
+#else
+/*
+ * Mac should always use a console channel, Unix should if it's trying to
+ */
+
+#define ShouldUseConsoleChannel(chan) (1)
+#endif
 
 /*
  *----------------------------------------------------------------------
  *
- * TkConsoleCreate --
+ * Tk_InitConsoleChannels --
  *
  * 	Create the console channels and install them as the standard
  * 	channels.  All I/O will be discarded until TkConsoleInit is
@@ -96,37 +206,95 @@ static Tcl_ChannelType consoleChannelType = {
  */
 
 void
-TkConsoleCreate()
+Tk_InitConsoleChannels(interp)
+    Tcl_Interp *interp;
 {
     Tcl_Channel consoleChannel;
 
-    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console0",
-	    (ClientData) TCL_STDIN, TCL_READABLE);
-    if (consoleChannel != NULL) {
-	Tcl_SetChannelOption(NULL, consoleChannel, "-translation", "lf");
-	Tcl_SetChannelOption(NULL, consoleChannel, "-buffering", "none");
+    /*
+     * Ensure that we are getting the matching version of Tcl.  This is
+     * really only an issue when Tk is loaded dynamically.
+     */
+
+    if (Tcl_InitStubs(interp, TCL_VERSION, 1) == NULL) {
+        return;
     }
-    Tcl_SetStdChannel(consoleChannel, TCL_STDIN);
-    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console1",
-	    (ClientData) TCL_STDOUT, TCL_WRITABLE);
-    if (consoleChannel != NULL) {
-	Tcl_SetChannelOption(NULL, consoleChannel, "-translation", "lf");
-	Tcl_SetChannelOption(NULL, consoleChannel, "-buffering", "none");
+
+    Tcl_MutexLock(&consoleMutex);
+    if (!consoleInitialized) {
+
+	consoleInitialized = 1;
+	
+	/*
+	 * check for STDIN, otherwise create it
+	 *
+	 * Don't do this check on the Mac, because it is hard to prevent
+	 * callbacks from the SIOUX layer from opening stdout & stdin, but
+	 * we don't want to use the SIOUX console.  Since the console is not
+	 * actually created till something is written to the channel, it is
+	 * okay to just ignore it here.
+	 *
+	 * This is still a bit of a hack, however, and should be cleaned up
+	 * when we have a better abstraction for the console.
+	 */
+
+	if (ShouldUseConsoleChannel(TCL_STDIN)) {
+	    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console0",
+		    (ClientData) TCL_STDIN, TCL_READABLE);
+	    if (consoleChannel != NULL) {
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-translation", "lf");
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-buffering", "none");
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-encoding", "utf-8");
+	    }
+	    Tcl_SetStdChannel(consoleChannel, TCL_STDIN);
+	}
+
+	/*
+	 * check for STDOUT, otherwise create it
+	 */
+	
+	if (ShouldUseConsoleChannel(TCL_STDOUT)) {
+	    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console1",
+		    (ClientData) TCL_STDOUT, TCL_WRITABLE);
+	    if (consoleChannel != NULL) {
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-translation", "lf");
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-buffering", "none");
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-encoding", "utf-8");
+	    }
+	    Tcl_SetStdChannel(consoleChannel, TCL_STDOUT);
+	}
+	
+	/*
+	 * check for STDERR, otherwise create it
+	 */
+	
+	if (ShouldUseConsoleChannel(TCL_STDERR)) {
+	    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console2",
+		    (ClientData) TCL_STDERR, TCL_WRITABLE);
+	    if (consoleChannel != NULL) {
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-translation", "lf");
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-buffering", "none");
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-encoding", "utf-8");
+	    }
+	    Tcl_SetStdChannel(consoleChannel, TCL_STDERR);
+	}
     }
-    Tcl_SetStdChannel(consoleChannel, TCL_STDOUT);
-    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console2",
-	    (ClientData) TCL_STDERR, TCL_WRITABLE);
-    if (consoleChannel != NULL) {
-	Tcl_SetChannelOption(NULL, consoleChannel, "-translation", "lf");
-	Tcl_SetChannelOption(NULL, consoleChannel, "-buffering", "none");
-    }
-    Tcl_SetStdChannel(consoleChannel, TCL_STDERR);
+    Tcl_MutexUnlock(&consoleMutex);
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * TkConsoleInit --
+ * Tk_CreateConsoleWindow --
  *
  *	Initialize the console.  This code actually creates a new
  *	application and associated interpreter.  This effectivly hides
@@ -142,16 +310,18 @@ TkConsoleCreate()
  */
 
 int 
-TkConsoleInit(interp)
+Tk_CreateConsoleWindow(interp)
     Tcl_Interp *interp;			/* Interpreter to use for prompting. */
 {
     Tcl_Interp *consoleInterp;
     ConsoleInfo *info;
     Tk_Window mainWindow = Tk_MainWindow(interp);
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 #ifdef MAC_TCL
     static char initCmd[] = "source -rsrc {Console}";
 #else
-    static char initCmd[] = "if [catch {source -rsrc console}] {source [file join $tk_library console.tcl]}";
+    static char initCmd[] = "if [catch {::wrap::source lib/tk$tk_version/console}] {source [file join $tk_library console.tcl]}";
 #endif
     
     consoleInterp = Tcl_CreateInterp();
@@ -169,7 +339,7 @@ TkConsoleInit(interp)
     if (Tk_Init(consoleInterp) != TCL_OK) {
 	goto error;
     }
-    gStdoutInterp = interp;
+    tsdPtr->gStdoutInterp = interp;
     
     /* 
      * Add console commands to the interp 
@@ -225,11 +395,15 @@ ConsoleOutput(instanceData, buf, toWrite, errorCode)
     int toWrite;			/* How many bytes to write? */
     int *errorCode;			/* Where to store error code. */
 {
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+
     *errorCode = 0;
     Tcl_SetErrno(0);
 
-    if (gStdoutInterp != NULL) {
-	TkConsolePrint(gStdoutInterp, (int) instanceData, buf, toWrite);
+    if (tsdPtr->gStdoutInterp != NULL) {
+	TkConsolePrint(tsdPtr->gStdoutInterp, (int) instanceData, buf, 
+                toWrite);
     }
     
     return toWrite;
@@ -373,9 +547,10 @@ ConsoleCmd(clientData, interp, argc, argv)
 {
     ConsoleInfo *info = (ConsoleInfo *) clientData;
     char c;
-    int length;
+    size_t length;
     int result;
     Tcl_Interp *consoleInterp;
+    Tcl_DString dString;
 
     if (argc < 2) {
 	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
@@ -388,23 +563,25 @@ ConsoleCmd(clientData, interp, argc, argv)
     result = TCL_OK;
     consoleInterp = info->consoleInterp;
     Tcl_Preserve((ClientData) consoleInterp);
+    Tcl_DStringInit(&dString);
+
     if ((c == 't') && (strncmp(argv[1], "title", length)) == 0) {
-	Tcl_DString dString;
-	
-	Tcl_DStringInit(&dString);
 	Tcl_DStringAppend(&dString, "wm title . ", -1);
 	if (argc == 3) {
 	    Tcl_DStringAppendElement(&dString, argv[2]);
 	}
 	Tcl_Eval(consoleInterp, Tcl_DStringValue(&dString));
-	Tcl_DStringFree(&dString);
     } else if ((c == 'h') && (strncmp(argv[1], "hide", length)) == 0) {
-	Tcl_Eval(info->consoleInterp, "wm withdraw .");
+	Tcl_DStringAppend(&dString, "wm withdraw . ", -1);
+	Tcl_Eval(consoleInterp, Tcl_DStringValue(&dString));
     } else if ((c == 's') && (strncmp(argv[1], "show", length)) == 0) {
-	Tcl_Eval(info->consoleInterp, "wm deiconify .");
+	Tcl_DStringAppend(&dString, "wm deiconify . ", -1);
+	Tcl_Eval(consoleInterp, Tcl_DStringValue(&dString));
     } else if ((c == 'e') && (strncmp(argv[1], "eval", length)) == 0) {
 	if (argc == 3) {
-	    Tcl_Eval(info->consoleInterp, argv[2]);
+	    result = Tcl_Eval(consoleInterp, argv[2]);
+	    Tcl_AppendResult(interp, Tcl_GetStringResult(consoleInterp),
+		    (char *) NULL);
 	} else {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
 		    " eval command\"", (char *) NULL);
@@ -416,6 +593,7 @@ ConsoleCmd(clientData, interp, argc, argv)
 		(char *) NULL);
         result = TCL_ERROR;
     }
+    Tcl_DStringFree(&dString);
     Tcl_Release((ClientData) consoleInterp);
     return result;
 }
@@ -446,7 +624,7 @@ InterpreterCmd(clientData, interp, argc, argv)
 {
     ConsoleInfo *info = (ConsoleInfo *) clientData;
     char c;
-    int length;
+    size_t length;
     int result;
     Tcl_Interp *otherInterp;
 
@@ -466,6 +644,7 @@ InterpreterCmd(clientData, interp, argc, argv)
     } else if ((c == 'r') && (strncmp(argv[1], "record", length)) == 0) {
    	Tcl_RecordAndEval(otherInterp, argv[2], TCL_EVAL_GLOBAL);
 	result = TCL_OK;
+	Tcl_ResetResult(interp);
     	Tcl_AppendResult(interp, otherInterp->result, (char *) NULL);
     } else {
 	Tcl_AppendResult(interp, "bad option \"", argv[1],
@@ -530,9 +709,13 @@ ConsoleEventProc(clientData, eventPtr)
 {
     ConsoleInfo *info = (ConsoleInfo *) clientData;
     Tcl_Interp *consoleInterp;
+    Tcl_DString dString;
     
     if (eventPtr->type == DestroyNotify) {
-        consoleInterp = info->consoleInterp;
+
+	Tcl_DStringInit(&dString);
+  
+	consoleInterp = info->consoleInterp;
 
         /*
          * It is possible that the console interpreter itself has
@@ -545,7 +728,9 @@ ConsoleEventProc(clientData, eventPtr)
             return;
         }
         Tcl_Preserve((ClientData) consoleInterp);
-	Tcl_Eval(consoleInterp, "tkConsoleExit");
+	Tcl_DStringAppend(&dString, "tkConsoleExit", -1);
+	Tcl_Eval(consoleInterp, Tcl_DStringValue(&dString));
+	Tcl_DStringFree(&dString);
         Tcl_Release((ClientData) consoleInterp);
     }
 }
@@ -603,7 +788,7 @@ TkConsolePrint(interp, devId, buffer, size)
     Tcl_DStringAppend(&output, buffer, size);
 
     Tcl_DStringInit(&command);
-    Tcl_DStringAppend(&command, cmd, strlen(cmd));
+    Tcl_DStringAppend(&command, cmd, (int) strlen(cmd));
     Tcl_DStringAppendElement(&command, output.string);
 
     consoleInterp = info->consoleInterp;
