@@ -21,6 +21,9 @@
 #if !defined(__WIN32__) && !defined(MAC_TCL)
 #include "tkUnixInt.h"
 #endif
+#if defined(__WIN32__)
+#include "tkWinInt.h"
+#endif
 
 
 typedef struct ThreadSpecificData {
@@ -137,7 +140,7 @@ static TkCmd commands[] = {
     {"canvas",		NULL,			Tk_CanvasObjCmd,	1, 1},
     {"checkbutton",	NULL,			Tk_CheckbuttonObjCmd,	1, 0},
     {"entry",		NULL,                   Tk_EntryObjCmd,		1, 0},
-    {"frame",		Tk_FrameCmd,		NULL,			1, 1},
+    {"frame",		NULL,			Tk_FrameObjCmd,		1, 1},
     {"label",		NULL,			Tk_LabelObjCmd,		1, 0},
     {"listbox",		Tk_ListboxCmd,		NULL,			1, 1},
     {"menubutton",	NULL,                   Tk_MenubuttonObjCmd,	1, 0},
@@ -146,7 +149,7 @@ static TkCmd commands[] = {
     {"scale",		NULL,	                Tk_ScaleObjCmd,		1, 0},
     {"scrollbar",	Tk_ScrollbarCmd,	NULL,			1, 1},
     {"text",		Tk_TextCmd,		NULL,			1, 1},
-    {"toplevel",	Tk_ToplevelCmd,		NULL,			0, 1},
+    {"toplevel",	NULL,			Tk_ToplevelObjCmd,	0, 1},
 
     /*
      * Misc.
@@ -243,6 +246,7 @@ static void		IMDestroyCallback _ANSI_ARGS_ ((XIM im, XPointer clientData,
  *
  *----------------------------------------------------------------------
  */
+#undef Tk_CreatePhotoImageFormat
 
 static Tk_Window
 CreateTopLevelWindow(interp, parent, name, screenName)
@@ -272,14 +276,14 @@ CreateTopLevelWindow(interp, parent, name, screenName)
 	 * Create built-in image types.
 	 */
     
-	Tk_CreateImageType(&tkBitmapImageType);
-	Tk_CreateImageType(&tkPhotoImageType);
+	Tk_CreateNewImageType(&tkBitmapImageType);
+	Tk_CreateNewImageType(&tkPhotoImageType);
     
 	/*
 	 * Create built-in photo image formats.
 	 */
     
-	Tk_CreatePhotoImageFormat(&tkImgFmtGIF);
+	Tk_CreateNewPhotoImageFormat(&tkImgFmtGIF);
 	Tk_CreatePhotoImageFormat(&tkImgFmtPPM);
 
 	/*
@@ -2964,10 +2968,22 @@ Initialize(interp)
 {
     char *p;
     int argc, code;
-    char **argv, *args[20];
+    char **argv;
+    Tcl_Obj *args[20];
     Tcl_DString class;
     ThreadSpecificData *tsdPtr;
+    static char version[] = TK_PATCH_LEVEL;
     
+#if defined(__WIN32__) && defined(STATIC_BUILD) && !defined(NO_STATIC_INIT)
+    /*
+     * If we are in a statically linked executable, then we need to
+     * explicitly initialize the Windows function tables here since
+     * DllMain() will not be invoked.
+     */
+
+    TkWinXInit(GetModuleHandle(NULL));
+#endif
+
     /*
      * Ensure that we are getting the matching version of Tcl.  This is
      * really only an issue when Tk is loaded dynamically.
@@ -2977,9 +2993,9 @@ Initialize(interp)
         return TCL_ERROR;
     }
 
-#ifdef USE_TCL_STUBS
-    Tcl_MainLoop = Tk_MainLoop;
-#endif
+    if (Tcl_SetMainLoop) {
+	Tcl_SetMainLoop(Tk_GetNumMainWindows);
+    }
 
     tsdPtr = (ThreadSpecificData *) 
 	Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
@@ -3133,14 +3149,14 @@ Initialize(interp)
      * using the information parsed from argv, if any.
      */
 
-    args[0] = "toplevel";
-    args[1] = ".";
-    args[2] = "-class";
-    args[3] = Tcl_DStringValue(&class);
+    args[0] = Tcl_NewStringObj("toplevel", -1);
+    args[1] = Tcl_NewStringObj(".", -1);
+    args[2] = Tcl_NewStringObj("-class", -1);
+    args[3] = Tcl_NewStringObj(Tcl_DStringValue(&class), -1);
     argc = 4;
     if (display != NULL) {
-	args[argc] = "-screen";
-	args[argc+1] = display;
+	args[argc] = Tcl_NewStringObj("-screen", -1);
+	args[argc+1] = Tcl_NewStringObj(display, -1);
 	argc += 2;
 
 	/*
@@ -3154,20 +3170,20 @@ Initialize(interp)
 	}
     }
     if (colormap != NULL) {
-	args[argc] = "-colormap";
-	args[argc+1] = colormap;
+	args[argc] = Tcl_NewStringObj("-colormap", -1);
+	args[argc+1] = Tcl_NewStringObj(colormap, -1);
 	argc += 2;
         colormap = NULL;
     }
     if (use != NULL) {
-	args[argc] = "-use";
-	args[argc+1] = use;
+	args[argc] = Tcl_NewStringObj("-use", -1);
+	args[argc+1] = Tcl_NewStringObj(use, -1);
 	argc += 2;
         use = NULL;
     }
     if (visual != NULL) {
-	args[argc] = "-visual";
-	args[argc+1] = visual;
+	args[argc] = Tcl_NewStringObj("-visual", -1);
+	args[argc+1] = Tcl_NewStringObj(visual, -1);
 	argc += 2;
         visual = NULL;
     }
@@ -3207,7 +3223,8 @@ Initialize(interp)
      * Provide Tk and its stub table.
      */
 
-    code = Tcl_PkgProvideEx(interp, "Tk", TK_VERSION, (ClientData) &tkStubs);
+    version[5] = 0;
+    code = Tcl_PkgProvideEx(interp, "Tk", version, (ClientData) &tkStubs);
     if (code != TCL_OK) {
 	goto done;
     }
