@@ -60,10 +60,12 @@ bind Text <Triple-1> {
     tkTextSelectTo %W %x %y
     catch {%W mark set insert sel.first}
 }
-bind Text <Quadruple-1> {
-    set tkPriv(selectMode) all
-    tkTextSelectTo %W %x %y
-    catch {%W mark set insert sel.first}
+catch {
+    bind Text <Quadruple-1> {
+	set tkPriv(selectMode) all
+	tkTextSelectTo %W %x %y
+	catch {%W mark set insert sel.first}
+    }
 }
 bind Text <Shift-1> {
     tkTextResetAnchor %W @%x,%y
@@ -78,9 +80,11 @@ bind Text <Triple-Shift-1>	{
     set tkPriv(selectMode) line
     tkTextSelectTo %W %x %y
 }
-bind Text <Quadruple-Shift-1> {
-    set tkPriv(selectMode) all
-    tkTextSelectTo %W %x %y
+catch {
+    bind Text <Quadruple-Shift-1> {
+	set tkPriv(selectMode) all
+	tkTextSelectTo %W %x %y
+    }
 }
 bind Text <B1-Leave> {
     set tkPriv(x) %x
@@ -386,14 +390,19 @@ bind Text <Meta-Delete> {
 	%W delete [tkTextPrevPos %W insert tcl_startOfPreviousWord] insert
     }
 }
-
-# Macintosh only bindings:
-
-# if text black & highlight black -> text white, other text the same
-if {![string compare $tcl_platform(platform) "macintosh"]} {
+switch $tcl_platform(platform) {
+  "unix" {
+    # input method configuration binding
+    if {[string compare [info commands imconfigure] {}]} {
+	bind Text <FocusIn> {catch {imconfigure %W}}
+    }
+  }
+  "macintosh" {
+  # if text black & highlight black -> text white, other text the same
 bind Text <FocusIn> {
-    %W tag configure sel -borderwidth 0
-    %W configure -selectbackground systemHighlight -selectforeground systemHighlightText
+	%W tag configure sel -borderwidth 0
+	%W configure -selectbackground systemHighlight \
+		-selectforeground systemHighlightText
 }
 bind Text <FocusOut> {
     %W tag configure sel -borderwidth 1
@@ -425,6 +434,7 @@ bind Text <Shift-Option-Down> {
 }
 
 # End of Mac only bindings
+ }
 }
 
 # A few additional bindings of my own.
